function [pfResults, BIBC]=LFThukaram(ep, busnum, basemva, basevoltage, busdat, linedat, BIBC)
% load flow analysis Sweep Thukaram Load Flow
% [1] D. Thukaram, H.M. Wijekoon Banda, J. Jerome, ...
% A robust three phase power flow algorithm for radial distribution systems, ...
% Electric Power Systems Research, 50 (1999) 227-236.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  Radial Distribution Power Flow                        
%                            from [1]                                    
%                              V2.0                                      
%                    22-01-2015  (last update)                           
%                               by                                       
%                  Joo Soares (joaps@isep.ipp.pt)                                  
%   GECAD, ISEP, 2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

P0=busdat(:,7)/basemva;
Q0=busdat(:,8)/basemva;
%Sg=(busdat(:,5)+busdat(:,6)*sqrt(-1))/basemva;
Sg=(busdat(:,5)+busdat(:,6)*1i)/basemva;

np=busdat(:,9);
nq=busdat(:,10);
rx=linedat(:,2:6);

global BIBC_global
%BIBC building
if isempty(BIBC_global)
    n=max(max(rx(:,1)), max(rx(:,2))); % bus number
    [nb ~]=size(rx);  % nb branch number
    %fromb=[];
    BIBC=zeros(nb,n-1);
    for i=1:nb
        fromb=[];
        tob=rx(i,2);
        BIBC(i,tob-1)=1;
        fromb=[fromb; rx(i,2)] ;
        %forKs=find(fromb==rx(i+1:nb,1));
        for k=i+1:nb
            %if isempty(find(fromb==rx(k,1)))
            if any(fromb==rx(k,1))
                tob=rx(k,2);
                BIBC(i,tob-1)=1;
                fromb=[fromb; rx(k,2)];
            end
        end
    end
    BIBC_global=BIBC;
else
    BIBC=BIBC_global;
end

% Voltage initial values
V=ones(busnum,1);
%I=zeros(n,1);
eps=1;
it=0;

% Get voltage specified for slack bus
V(1)=busdat(1,3);

while eps>ep && it<100
    it=it+1;
    Vpr=V;
    %        for ii=1:n
    %         Pa(ii)=P0(ii)*(abs(V(ii))^np(ii));
    %         Qa(ii)=Q0(ii)*(abs(V(ii))^nq(ii));
    %         S(ii)=Pa(ii)+j*Qa(ii);
    %        end
    % equivalente to above using code vectorization
    Pa=P0.*(abs(V).^np);
    Qa=Q0.*(abs(V).^np);
    S=Pa+1i.*Qa;
    
    %     for ii=1:n
    %         I(ii)=conj((S(ii)-Sg(ii))/V(ii));
    %     end
    % equivalente to above using code vectorization
    I=conj((S-Sg)./V);
    
    Inew=I;
    Inew(1)=[];
    B=BIBC*Inew;
    Vnew=V;
    Vnew(1)=[];
    Snew=conj(B).*Vnew;
    
    %Bus voltage calculation
    for i=1:busnum-1
        s=rx(i,1);
        r=rx(i,2);
        rz=rx(i,3);
        xz=rx(i,4);
        V(r)=V(s)-B(r-1)*(rz+1i*xz);
    end
    eps=max(abs(V-Vpr));
end
% %%%  load flow analysis  Sweep Thukaram Load Flow
%%% load flow output data
%it;
%Vm=abs(V);
%Va=angle(V)*180/pi;
%         LineCur=B*basemva*1000/basevoltage;
%         SLoad=S.'*basemva;
%         Lineloss=abs(B).^2.*(rx(:,3)+j*rx(:,4))*basemva;
%         Sline=Snew*basemva+Lineloss;
pfResults.it=it;
pfResults.Vm=abs(V);
pfResults.Va=angle(V)*180/pi;
pfResults.LineCur=B*basemva*1000/basevoltage;
pfResults.Sload=S.'*basemva;
pfResults.Lineloss=abs(B).^2.*(rx(:,3)+1i*rx(:,4))*basemva;
pfResults.Sline=Snew*basemva+pfResults.Lineloss;

return